import sys
import time
import grovepi

DISPLAY_RGB_ADDR = 0x30
DISPLAY_TEXT_ADDR = 0x3e

PIR_SENSOR_A_PIN = 3
BUZZER_PIN = 4

if sys.platform == 'uwp':
    import winrt_smbus as smbus
    bus = smbus.SMBus(1)
else:
    import smbus
    import RPi.GPIO as GPIO
    rev = GPIO.RPI_REVISION
    if rev == 2 or rev == 3:
        bus = smbus.SMBus(1)
    else:
        bus = smbus.SMBus(0)


def init_pins():
    grovepi.pinMode(PIR_SENSOR_A_PIN, "INPUT")
    grovepi.pinMode(BUZZER_PIN, "OUTPUT")


def buzz():
    grovepi.digitalWrite(BUZZER_PIN, 1)
    time.sleep(0.5)
    grovepi.digitalWrite(BUZZER_PIN, 0)


def is_motion():
    return grovepi.digitalRead(PIR_SENSOR_A_PIN)


def setRGB(r, g, b):
    """
    Controls the Grove LCD backlight color by writing to the
    device at DISPLAY_RGB_ADDR.
    """
    bus.write_byte_data(DISPLAY_RGB_ADDR, 0x04, 0x15)
    
    bus.write_byte_data(DISPLAY_RGB_ADDR, 0x06, r)
    bus.write_byte_data(DISPLAY_RGB_ADDR, 0x07, g)
    bus.write_byte_data(DISPLAY_RGB_ADDR, 0x08, b)


def textCommand(cmd):
    """
    Sends a command byte to the LCD text command register at DISPLAY_TEXT_ADDR (0x80).
    Used internally by setText() to configure display settings or move the cursor.
    """
    bus.write_byte_data(DISPLAY_TEXT_ADDR, 0x80, cmd)


def setText(text):
    """
    Clears the display, configures it for 2-line mode, then writes
    up to 32 characters. If it hits 16 chars or a newline,
    it moves to the second line. Excess text is ignored.
    """
    # Clear display
    textCommand(0x01)
    time.sleep(0.05)
    # Display on, no cursor
    textCommand(0x08 | 0x04)
    # 2-line mode
    textCommand(0x28)
    time.sleep(0.05)

    count = 0
    row = 0
    for c in text:
        if c == '\n' or count == 16:
            # Move to next line
            count = 0
            row += 1
            if row == 2:  # Only 2 lines available
                break
            textCommand(0xc0)  # Move cursor to second line
            if c == '\n':
                continue
        count += 1
        # Write character to the LCD data register (0x40)
        bus.write_byte_data(DISPLAY_TEXT_ADDR, 0x40, ord(c))
