from datetime import datetime
from sqlalchemy import Integer, String, DateTime
from sqlalchemy.orm import DeclarativeBase
from sqlalchemy.orm import mapped_column
from uuid import uuid4

ENGLISH_LANGUAGE = "en"
WELSH_LANGUAGE = "cy"

DEFAULT_LANGUAGE = ENGLISH_LANGUAGE

class Base(DeclarativeBase):
    pass


class Customer(Base):
    __tablename__ = "customers"

    id = mapped_column(String, primary_key=True)
    first_name = mapped_column(String(50), nullable=False)
    email = mapped_column(String, nullable=False)
    locale = mapped_column(String, nullable=False, default=DEFAULT_LANGUAGE)


class Product(Base):
    __tablename__ = "products"

    id = mapped_column(String, primary_key=True)
    name = mapped_column(String(50), nullable=False)
    recycling_type = mapped_column(Integer, nullable=False)
    reward_value = mapped_column(Integer, nullable=False)


class Return(Base):
    __tablename__ = "returns"

    id = mapped_column(String, primary_key=True, default=lambda: str(uuid4()))
    customer_id = mapped_column(String, nullable=False)
    product_id = mapped_column(String, nullable=False)
    return_date = mapped_column(DateTime, nullable=False, default=datetime.utcnow)
    reward_value = mapped_column(Integer, nullable=False)
