import paho.mqtt.client as mqtt
import json
# -----------------------------------------------------------------------------
# 2. ThingsBoard Configuration
# -----------------------------------------------------------------------------
#  - We connect to a ThingsBoard server for storing and visualizing sensor data.
#  - 'thingsboard.cs.cf.ac.uk' is the server address for your institution.
#  - ACCESS_TOKEN is a unique string that authenticates this device to ThingsBoard.
#    Ensure you use your own token to post data to your own ThingsBoard dashboard.
# -----------------------------------------------------------------------------
THINGSBOARD_HOST = 'thingsboard.cs.cf.ac.uk'
ACCESS_TOKEN = 'zHObHAhH5uTIwRrT8GF7'  # Replace with your actual token as needed

# -----------------------------------------------------------------------------
# 3. MQTT Callbacks
# -----------------------------------------------------------------------------
#  - These callback functions are automatically called by the paho-mqtt library 
#    in response to various MQTT events (publishing, receiving messages, connecting).
# -----------------------------------------------------------------------------
def on_publish(client, userdata, result):
    """
    Called whenever a message is successfully published to the MQTT broker.
    Prints 'Success' to confirm the publish event, helping you verify 
    that data actually reached the server.
    """
    print("Success")

def on_message(client, userdata, msg):
    """
    Called when the client receives a message on a subscribed topic.
    For this script, we listen for RPC requests that instruct the device
    to toggle the buzzer. The message payload is in JSON format, which we decode.
    """
    print('Topic: ' + msg.topic + '\nMessage: ' + str(msg.payload))
    data = json.loads(msg.payload)
    
    # If the method is "setValue", we interpret "params" to be either True or False,
    # which determines the buzzer's ON/OFF state.
    if data['method'] == 'setValue':
        print(data['params'])
        buzzer_state['State'] = data['params']
        # The grovepi.digitalWrite function sends a digital HIGH or LOW to the buzzer pin
        # depending on whether we pass True (HIGH) or False (LOW).
        grovepi.digitalWrite(buzzer, data['params'])

def on_connect(client, userdata, flags, rc, *extra_params):
    """
    Called after the client attempts to connect to the MQTT broker.
    rc (result code) = 0 means a successful connection.
    This is a good place to log or print a message indicating the connection status.
    """
    print('Connected with result code ' + str(rc))
    

# -----------------------------------------------------------------------------
# 5. MQTT Client Creation and Configuration
# -----------------------------------------------------------------------------
#  - The paho.mqtt library needs a client instance to manage connections and 
#    message events. We'll attach our callback functions and connect to the 
#    ThingsBoard server using our access token.
# -----------------------------------------------------------------------------
client = mqtt.Client()

def init_mqtt():
    # Provide the ACCESS_TOKEN for device authentication to ThingsBoard.
    client.username_pw_set(ACCESS_TOKEN)

    # Bind our callbacks for connect, publish, and message events.
    client.on_connect = on_connect
    client.on_publish = on_publish
    client.on_message = on_message

    # Connect to the ThingsBoard server on port 1883 (standard MQTT).
    # The third parameter (60) is the keepalive interval in seconds.
    client.connect(THINGSBOARD_HOST, 1883, 60)

    # Subscribe to RPC commands from ThingsBoard to let remote users
    # toggle the buzzer by sending "setValue" method calls.
    client.subscribe('v1/devices/me/rpc/request/+')

    # Start the MQTT network loop in a separate thread, so the script can continue
    # running and processing sensor data while maintaining an MQTT connection.
    client.loop_start()


def publish_return(new_return):
    payload = {
        event_type: "return",
        data: new_return,
    }
    client.publish('v1/devices/me/telemetry', json.dumps(payload), qos=1)

